# -*- coding: utf-8 -*-
# Copyright 2009 Intevation GmbH, Germany, <info@intevation.de>
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#


import sys
import os.path
import re
from xml.dom.minidom import parse

from formed.meta.data import RelevantVisitor


ANNOTATION_DIV_ID = \
          re.compile("([a-zA-Z0-9_\-]+):([a-zA-Z0-9_\-]+):([a-zA-Z0-9_\-,]+)")


class Annotation:

    def __init__(self, kind, msg):
        self.kind = kind
        self.msg = msg

    def __str__(self):
        return repr(self.kind) + ": " + repr(self.msg)

    def __repr__(self):
        return self.__str__()


class Annotations(object):

    def __init__(self, annotations_file):
        self.annotations = {}
        self.load(annotations_file)

    def load(self, annotations_file):
        if not os.path.exists(annotations_file):
            return

        annotations = self.annotations

        dom = None
        try:
            dom = parse(annotations_file)

            for div in dom.getElementsByTagName("div"):
                id = div.getAttribute("id")
                if not id:
                    continue
                m = ANNOTATION_DIV_ID.match(id)
                if not m:
                    continue
                id, kind = m.group(1), m.group(2)
                items = [item for item in m.group(3).split(",") if item]
                if not items:
                    continue
                div.setAttribute("id", id)

                msg = unicode(div.toxml())

                for item in items:
                    annons = annotations.setdefault(item, [])
                    annons.append(Annotation(kind, msg))
        finally:
            if dom:
                dom.unlink()

    def decorate(self, document):
        def callback(nc):
            try:
                nc.setAnnotations(self.annotations[nc.getName()])
            except KeyError:
                pass
        visitor = RelevantVisitor(callback)
        document.visit(visitor.visitor)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
