# -*- coding: utf-8 -*-

import logging
import formencode

import paste

from pylons import request, tmpl_context as c
from pylons.controllers.util import redirect_to
from mpulsweb.lib.translation import _, ungettext

from mpulsweb.lib.base import BaseController, render, session
from mpulsweb.lib.validators import UploadCaseDocumentForm
from mpulsweb.lib.security import checkRole
import mpulsweb.lib.helpers as h

from mpulsweb.model.document import Document, deleteDocument


log = logging.getLogger(__name__)

OK_TO_CONTINUE = _(u"Please click OK to continue.")

SET_ATTACHMENT_SUCCESS_HEADER = _(u"Attachment added!")

SET_ATTACHMENT_FAILED_HEADER = _(u"No attachment added!")
SET_ATTACHMENT_FAILED_TEXT = _(u"""Please make sure that the path is correct. Attachments
can be viewed directly via the "Browse" button""") + " " + OK_TO_CONTINUE

SET_ATTACHMENT_SUCCESS_DELETE_HEADER = _(u"Attachment deleted!""")

SET_ATTACHMENT_CONFIRM_DELETE_HEADER = _(u"Delete attachment?")
SET_ATTACHMENT_CONFIRM_DELETE_TEXT = _(u"Do you want to irrevocably delete the attachment?")


class CasedocumentController(BaseController):

    @checkRole(('admin_ka', 'cm_ka'))
    def index(self, id):
        return self.overview(id)

    @checkRole(('admin_ka', 'cm_ka'))
    def overview(self, id):
        id = self._checkInt(id)
        case = self._loadCase(id)
        c.files = case.getDocuments()
        return render('/documents/case_overview.mako')

    @checkRole('cm_ka')
    def new(self):
        return render('/documents/case_new.mako')

    @checkRole('cm_ka')
    def newAction(self):
        validator = UploadCaseDocumentForm()
        try:
            result = validator.to_python(request.params)
            doc = Document()
            filename = result['file']['filename']
            content = result['file']['content']
            case = result['case']
            if result['name']:
                # Make sure the new name has a (3 letter) extension
                if '.' not in result['name'][-4:]:
                    extension = filename.rpartition('.')[2]
                    filename = result['name'] + '.' + extension
                else:
                    filename = result['name']
            doc.create(filename, content, case)
            c.dialog_title = SET_ATTACHMENT_SUCCESS_HEADER
            c.dialog_text = OK_TO_CONTINUE
            c.url_ok = h.url_for(controller="/casedocument", action="index",
                                 id=case)
            return render('/documents/dialogs/success_attachment.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            log.debug(c.form_result)
            form = render('/documents/case_new.mako')
            return formencode.htmlfill.render(form,
                                              defaults=c.form_result,
                                              auto_insert_errors=False,
                                              errors=c.form_errors)

    @checkRole(('admin_ka', 'cm_ka'))
    def show(self, id):
        id = self._checkInt(id)
        try:
            document = Document(id)
            document.load(True)
            data = paste.fileapp.DataApp(document.data,
                                         content_type=document.mime,
                                         cache_control='must-revalidate',
                                         pragma='public')
            return data(request.environ, self.start_response)
        except Exception, e:
            log.exception(e)
            redirect_to(controller="case", action="documents")

    @checkRole('cm_ka')
    def delete(self, id, confirmed):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            try:
                deleteDocument(id, True)
            except Exception, e:
                log.exception(e)
            c.url_ok = h.url_for(controller="/casedocument", action="index" ,
                                 id=session['case'].id)
            c.success_for = SET_ATTACHMENT_SUCCESS_DELETE_HEADER
            c.success_text = OK_TO_CONTINUE
            return render('/casemanagement/dialogs/'
                          'success_delete_attachment.mako')
        else:
            c.context = "../main.mako"
            c.confirm_for = SET_ATTACHMENT_CONFIRM_DELETE_HEADER
            c.question = SET_ATTACHMENT_CONFIRM_DELETE_TEXT
            c.url_yes = h.url_for(controller='/casedocument', action='delete',
                                  id=id, confirmed=1)
            c.url_no = h.url_for(controller="/casedocument", action="index",
                                 id=session['case'].id)
            return render('/casemanagement/dialogs/'
                          'confirm_delete_attachment.mako')
