# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging
import formencode

from mpulsweb.lib.translation import _

import paste

from mpulsweb.lib.base import BaseController, c, h, redirect_to, render, request
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.validators import UploadGlobalDocumentForm
from mpulsweb.model.document import Document, listDocuments, deleteDocument

OK_TO_CONTINUE = _("Please click OK to continue.")

SET_DOCUMENT_SUCCESS_HEADER = _(u"Document added!")
SET_DOCUMENT_FAILED_HEADER  = _(u"No document added!")
SET_DOCUMENT_FAILED_TEXT    = _(u"""Please make sure that the path is correct. Documents
can be viewed directly via the "Browse" button""") + " " + OK_TO_CONTINUE

SET_DOCUMENT_SUCCESS_DELETE_HEADER = _(u"Document deleted!")
SET_DOCUMENT_CONFIRM_DELETE_HEADER = _(u"Delete document?")
SET_DOCUMENT_CONFIRM_DELETE_TEXT   = _(u"Do you want to delete the document and irrevocably lose the data?")
SET_ATTACHMENT_SUCCESS_HEADER      = _(u"Attachment added!")
SET_ATTACHMENT_SUCCESS_TEXT        = OK_TO_CONTINUE
SET_ATTACHMENT_FAILED_HEADER       = _(u"No attachment added!")
SET_ATTACHMENT_FAILED_TEXT         = _(u"""Please make sure that the path is correct. Attachments
can be viewed directly via the "Browse" button""") + " " + OK_TO_CONTINUE
SET_ATTACHMENT_SUCCESS_DELETE_HEADER = _(u"Attachment deleted!""")
SET_ATTACHMENT_SUCCESS_DELETE_TEXT   = OK_TO_CONTINUE
SET_ATTACHMENT_CONFIRM_DELETE_HEADER = _(u"Delete attachment?")
SET_ATTACHMENT_CONFIRM_DELETE_TEXT   = _(u"Do you want to irrevocably delete the attachment?")

log = logging.getLogger(__name__)

class DocumentController(BaseController):

    @checkRole(('admin_ka', 'cm_ka'))
    def index(self):
        redirect_to(action='globalOverview')

    @checkRole(('admin_ka', 'cm_ka'))
    def globalOverview(self):
        try:
            c.files = listDocuments()
        except Exception, e:
            log.exception(e)
            raise
        return render('/documents/global_overview.mako')

    @checkRole('admin_ka')
    def globalUpload(self):
        return render('/documents/global_upload.mako')

    @checkRole('admin_ka')
    def globalUploadAction(self):
        validator = UploadGlobalDocumentForm()
        try:
            result = validator.to_python(request.params)
            doc = Document()
            filename = result['file']['filename']
            content = result['file']['content']
            if result['name']:
                filename = result['name']
            doc.create(filename, content)
            c.dialog_title = SET_DOCUMENT_SUCCESS_HEADER
            c.dialog_text = OK_TO_CONTINUE
            c.url_ok = h.url_for(controller="document",
                                 action="globalOverview")
            return render('/documents/dialogs/success_documents.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            log.debug(c.form_result)
            form = render('/documents/global_upload.mako')
            return formencode.htmlfill.render(form,
                                              defaults=c.form_result,
                                              auto_insert_errors=False,
                                              errors=c.form_errors)

    @checkRole(('admin_ka', 'cm_ka'))
    def globalShow(self, id):
        id = self._checkInt(id)
        try:
            document = Document(id)
            document.load()
            data = paste.fileapp.DataApp(document.data,
                                         content_type=document.mime,
                                         cache_control='must-revalidate',
                                         pragma='public')
            return data(request.environ, self.start_response)
        except Exception, e:
            log.exception(e)
            redirect_to(controller="case", action="documents", id=case)

    @checkRole('admin_ka')
    def globalDelete(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            try:
                deleteDocument(id)
            except Exception, e:
                log.exception(e)
                return self.globalOverview()
            c.url_ok = h.url_for(controller="/document",
                                 action="globalOverview")
            c.dialog_title = SET_DOCUMENT_SUCCESS_DELETE_HEADER
            c.dialog_text = OK_TO_CONTINUE
            return render('/documents/dialogs/delete_document_success.mako')
        else:
            c.context = "../main.mako"
            c.dialog_title = SET_DOCUMENT_CONFIRM_DELETE_HEADER
            c.dialog_text = SET_DOCUMENT_CONFIRM_DELETE_TEXT
            c.url_yes = h.url_for(controller='/document',
                                  action='globalDelete', id=id, confirmed=1)
            c.url_no = h.url_for(controller='/document',
                                 action='globalOverview')
            return render('/documents/dialogs/confirm.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
