# -*- coding: utf-8 -*-
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

import logging
import datetime

from pylons import app_globals as g

from mpulsweb.lib.helpers import timedelta_in_minutes
from mpulsweb.lib.db import db

log = logging.getLogger(__name__)

class LogbookEvaluation:

    """Zeitaufwände aus dem Logbuch aller Fälle"""

    def __init__(self, id):
        self.id = id
        
        self.sql = """SELECT
            sum(coalesce(l.dauer, '0 second'::interval)) AS dauer,
            count(l.id) AS anzahl, l.art, m.id
            FROM ka_logbuch_tbl_view l
            JOIN master_tbl_view m
            ON m.id = l.master_id
            WHERE l.master_id = %(id)s
            AND l.art <> '0'
            GROUP BY l.art, m.id
            ORDER BY l.art
        """ % {'id': id}
        
        self.sql_typ = """SELECT
            sum(coalesce(l.dauer, '0 second'::interval)) AS dauer,
            count(l.id) AS anzahl, l.%(table)s, m.id
            FROM ka_logbuch_tbl_view l
            JOIN master_tbl_view m
            ON m.id = l.master_id
            WHERE l.master_id = %(id)s
            AND l.art != 0
            GROUP BY l.%(table)s, m.id
            ORDER BY l.%(table)s
        """ % {'table': 'typ', 'id': id}

    def perform(self):
        unique_cases = []
        result = {}
        for num, category in enumerate(g.mpuls_config.get('logbook',
                                                          'categories')):
            for item in category.get('items'):
                result[item] = [0, 0]
            result['sum_cat%s' % num] = [0, 0]
        result['sum_all']= [0, 0]
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                cur.execute(self.sql)
                count = 0
                while True:
                    row = cur.fetchone()
                    if not row:
                        break

                    if row[3] not in unique_cases:
                        count += 1
                        unique_cases.append(row[3])

                    duration = int(timedelta_in_minutes(row[0]))
                    num = row[1]
                    type = str(row[2])

                    result[type][0] += num
                    result[type][1] += duration

                    # Search category of the item and sum num and duration
                    for cnum, category in enumerate(
                        g.mpuls_config.get('logbook', 'categories')):

                        if type in category.get('items'):
                            result['sum_cat%s' % cnum][0] += num
                            result['sum_cat%s' % cnum][1] += int(duration)
                            break

                    result['sum_all'][0] += num
                    result['sum_all'][1] += int(duration)
                return result
            except StandardError, e:
                log.exception(e)
                raise
        finally:
            db.recycleConnection(conn, cur)
    
    
    def perform_type_evaluation(self):
        """ Perform an evaluation on the "typ" field of logbooks"""
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(self.sql_typ)
            
            rows = cur.fetchall()
            
            # Initiate statistical data. Then iterate thorugh the data to
            # generate the real ones. Bit ugly.
            result = {'total': [], 'types':[]}
            total = {'amount': 0, 'time': 0, 'percent': 0}
            
            for row in rows:
                total['amount'] = total['amount'] + row[1]
                total['time'] = total['time'] + timedelta_in_minutes(row[0])
            
            for row in rows:
                if total['time']: # don't count if no time given
                    percent = timedelta_in_minutes(row[0]) / total['time'] * 100.0
                    result['types'].append({'typ': row[2],
                                'time' : timedelta_in_minutes(row[0]),
                                'percent': percent,
                                'amount' : row[1]})
                    total['percent'] += percent
            result['total'] = total
            return result
        finally:
            db.recycleConnection(conn, cur)


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
