"""MPuls extensions for session data"""

from pylons import session


class MPulsSession:

    """Extends the pylons session by prefixing all keys with an
    application specific string.

    This way, when multiple MPuls applications are run in the same
    server do no interfere with each other, even when they share the
    session storage.  Derived classes may override methods, especially
    translate_key to let application share some session entries.
    """

    # The prefix to add to the keys.
    key_prefix = "mpuls"

    def __init__(self):
        pass

    def translate_key(self, key):
        """Return the actual session key for the key.
        All methods that handle keys, use this method to translate the
        keys given to the function into the key to be used when
        accessing the pylons session object.
        In this default implementation the actual session key is key
        prefixed with self.key_prefix and a period ('.').  Derived
        classes should override this method if they want to treat some
        keys differently.
        """
        return self.key_prefix + "." + key

    def __getitem__(self, key):
        return session[self.translate_key(key)]

    def __setitem__(self, key, value):
        session[self.translate_key(key)] = value

    def __delitem__(self, key):
        del session[self.translate_key(key)]

    def get(self, key, default=None):
        return session.get(self.translate_key(key), default)

    def has_key(self, key):
        return session.has_key(self.translate_key(key))

    def save(self):
        session.save()

    def delete(self):
        session.delete()

    def invalidate(self):
        session.invalidate()

