BEGIN;

-- 
-- Repeat Group fuer Vereinbarungen
-- 

ALTER TABLE master_tbl DISABLE TRIGGER USER;


SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/06_rg_vereinbarungen.sql', current_timestamp);


-- Create a new function to generate RG-entries.
-- This is the original function, without the DS IS check and exception
CREATE OR REPLACE FUNCTION create_rg_vereinbarungen_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        --RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_vereinbarungen_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_vereinbarungen_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_vereinbarungen_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_vereinbarungen_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


-- Uebernahme der alten Eintraege in eine Repeat Group
--
-- Fuer alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
  rg_count INTEGER;
  uuid TEXT;
BEGIN
  FOR r in SELECT *
  FROM master_tbl LOOP
  
      -- Zunaechst alle RGs loeschen
      DELETE from rg_vereinbarungen_tbl WHERE master_id = r.id;

      -- Vereinbarung_a -> vereinbarung_mit: "junger Mensch", ID: 1
      IF r.vereinb_a IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_a,
	       vereinbarung_mit = 1
            WHERE id = rgid;
      END IF;

      -- Vereinbarung_b -> vereinbarung_mit: "Eltern/Sorgeberechtigte", ID: 2
      IF r.vereinb_b IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_b,
	       vereinbarung_mit = 2
            WHERE id = rgid;
      END IF;

      -- Vereinbarung_c -> vereinbarung_mit: "Koordinierungsstelle", ID: 3
      IF r.vereinb_c IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_c,
	       vereinbarung_mit = 3
            WHERE id = rgid;
      END IF;

      -- Vereinbarung_d -> vereinbarung_mit: "Lehrerin/Lehrer", ID: 4
      IF r.vereinb_d IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_d,
	       vereinbarung_mit = 4
            WHERE id = rgid;
      END IF;

      -- Vereinbarung_e -> vereinbarung_mit: "Jugendamt", ID: 5
      IF r.vereinb_e IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_e,
	       vereinbarung_mit = 5
            WHERE id = rgid;
      END IF;


      -- Vereinbarung_f -> vereinbarung_mit: "Sonstiges", ID: -2
      IF r.vereinb_f IS NOT NULL
        OR r.vereinb_f_t IS NOT NULL
      THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
            UPDATE rg_vereinbarungen_tbl set
	       vereinbarung_text = r.vereinb_f,
	       vereinbarung_sonstige = r.vereinb_f_t,
	       vereinbarung_mit = -2
            WHERE id = rgid;
      END IF;


      -- Falls keine RGs angelegt wurden, eine neue erstellen
      SELECT COUNT(id) INTO rg_count FROM rg_vereinbarungen_tbl WHERE master_id  = r.id;
      IF rg_count = 0 THEN
      	uuid = generate_uuidgen();
        rgid = create_rg_vereinbarungen_tbl(r.id, uuid);
      END IF;


  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
SELECT insert_rg();
SET ROLE :db_adm;
DROP FUNCTION insert_rg();




-- Restore old function with its corresponding exception
CREATE OR REPLACE FUNCTION create_rg_vereinbarungen_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_vereinbarungen_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_vereinbarungen_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_vereinbarungen_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_vereinbarungen_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

-- ROLLBACK;
COMMIT;
