BEGIN;

-- 
-- Repeat Group fuer Ansprechpartner
-- 

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/08_rg_ansprechpartner.sql', current_timestamp);


-- Create a new function to generate RG-entries.
-- This is the original function, without the DS IS check and exception
CREATE OR REPLACE FUNCTION create_rg_ansprechpartner_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        --RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_ansprechpartner_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_ansprechpartner_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_ansprechpartner_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_ansprechpartner_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;



-- Uebernahme der alten Eintraege in eine Repeat Group
--
-- Fuer alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
  rg_count INTEGER;
  uuid TEXT;
BEGIN
  FOR r in SELECT *
  FROM master_tbl LOOP
  
      -- Zunaechst alle RGs loeschen
      DELETE from rg_ansprechpartner_tbl WHERE master_id = r.id;

      -- Schule, ID: 1
      IF r.kd_schule_name_1 IS NOT NULL
	OR r.kd_schule_strasse_1 IS NOT NULL
	OR r.kd_schule_strassenr_1 IS NOT NULL
	OR r.kd_schule_plz_1 IS NOT NULL
	OR r.kd_schule_ort_1 IS NOT NULL
 	OR r.kd_schule_vorwahl_1 IS NOT NULL
 	OR r.kd_schule_telefonnummer_1 IS NOT NULL
	OR r.kd_schule_email_1 IS NOT NULL THEN
         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 1,
            ansprechpartner_name = r.kd_schule_name_1,
            ansprechpartner_strasse = r.kd_schule_strasse_1,
            ansprechpartner_strassenr = r.kd_schule_strassenr_1,
            ansprechpartner_plz = r.kd_schule_plz_1,
            ansprechpartner_ort = r.kd_schule_ort_1,
            ansprechpartner_vorwahl = r.kd_schule_vorwahl_1,
            ansprechpartner_telefonnummer = r.kd_schule_telefonnummer_1,
            ansprechpartner_email = r.kd_schule_email_1
	 WHERE id = rgid;
      END IF;

      -- Bezugslehrer/Bezugslehrerin, ID: 2
      IF r.bezugs_name_1 IS NOT NULL
            OR r.bezugs_vorname_1 IS NOT NULL
            OR r.bezugs_vorwahl_1 IS NOT NULL
            OR r.bezugs_telefonnummer_1 IS NOT NULL
            OR r.bezugs_email_1 IS NOT NULL THEN

         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 2,
            ansprechpartner_name = array_to_string(ARRAY[r.bezugs_name_1, r.bezugs_vorname_1], ', '),
            ansprechpartner_vorwahl = r.bezugs_vorwahl_1,
            ansprechpartner_telefonnummer = r.bezugs_telefonnummer_1,
            ansprechpartner_email = r.bezugs_email_1
	 WHERE id = rgid;
      END IF;
      
     -- Jugendamt, ID: 3
      IF r.kd_jugendamt_kundennummer_1 IS NOT NULL
            OR r.kd_jugendamt_strassenr_1 IS NOT NULL
            OR r.kd_jugendamt_plz_1 IS NOT NULL
            OR r.kd_jugendamt_ort_1 IS NOT NULL
            OR r.kd_jugendamt_strasse_1 IS NOT NULL
            OR r.kd_jugendamt_strassenr_1 IS NOT NULL
            OR r.kd_jugendamt_vorwahl_1 IS NOT NULL
            OR r.kd_jugendamt_telefonnummer_1 IS NOT NULL
            OR r.kd_jugendamt_email_1 IS NOT NULL THEN
         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 3,
            -- Es gibt kein Feld fuer Jugendamtname in der alten Tabelle
            -- ansprechpartner_name = '',
	    ansprechpartner_kundennummer = r.kd_jugendamt_kundennummer_1,
            ansprechpartner_ort = r.kd_jugendamt_ort_1,
            ansprechpartner_plz = r.kd_jugendamt_plz_1,
	    ansprechpartner_strasse = r.kd_jugendamt_strasse_1,
            ansprechpartner_strassenr = r.kd_jugendamt_strassenr_1,
            ansprechpartner_vorwahl = r.kd_jugendamt_vorwahl_1,
            ansprechpartner_telefonnummer = r.kd_jugendamt_telefonnummer_1,
            ansprechpartner_email = r.kd_jugendamt_email_1
	 WHERE id = rgid;
      END IF;

      -- weitere Institutionen 1, ID: 0
      IF r.kd_institution_name_1 IS NOT NULL
           OR r.kd_institution_kundennummer_1 IS NOT NULL
           OR r.kd_institution_strasse_1 IS NOT NULL
           OR r.kd_institution_strassenr_1 IS NOT NULL
           OR r.kd_institution_plz_1 IS NOT NULL
           OR r.kd_institution_ort_1 IS NOT NULL
           OR r.kd_institution_vorwahl_1 IS NOT NULL
           OR r.kd_institution_telefonnummer_1 IS NOT NULL
           OR r.kd_institution_email_1 IS NOT NULL THEN
         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 0,
            ansprechpartner_name = r.kd_institution_name_1,
	    ansprechpartner_kundennummer = r.kd_institution_kundennummer_1,
	    ansprechpartner_strasse = r.kd_institution_strasse_1,
	    ansprechpartner_strassenr = r.kd_institution_strassenr_1,
            ansprechpartner_plz = r.kd_institution_plz_1,
            ansprechpartner_ort = r.kd_institution_ort_1,
	    ansprechpartner_vorwahl = r.kd_institution_vorwahl_1,
	    ansprechpartner_telefonnummer = r.kd_institution_telefonnummer_1,
	    ansprechpartner_email = r.kd_institution_email_1
	 WHERE id = rgid;
      END IF;
      
      -- weitere Institutionen 2, ID: 0
      IF r.kd_institution_name_2 IS NOT NULL
           OR r.kd_institution_kundennummer_2 IS NOT NULL
           OR r.kd_institution_strasse_2 IS NOT NULL
           OR r.kd_institution_strassenr_2 IS NOT NULL
           OR r.kd_institution_plz_2 IS NOT NULL
           OR r.kd_institution_ort_2 IS NOT NULL
           OR r.kd_institution_vorwahl_2 IS NOT NULL
           OR r.kd_institution_telefonnummer_2 IS NOT NULL
           OR r.kd_institution_email_2 IS NOT NULL THEN

         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 0,
            ansprechpartner_name = r.kd_institution_name_2,
	    ansprechpartner_kundennummer = r.kd_institution_kundennummer_2,
	    ansprechpartner_strasse = r.kd_institution_strasse_2,
	    ansprechpartner_strassenr = r.kd_institution_strassenr_2,
            ansprechpartner_plz = r.kd_institution_plz_2,
            ansprechpartner_ort = r.kd_institution_ort_2,
	    ansprechpartner_vorwahl = r.kd_institution_vorwahl_2,
	    ansprechpartner_telefonnummer = r.kd_institution_telefonnummer_2,
	    ansprechpartner_email = r.kd_institution_email_2
	 WHERE id = rgid;
      END IF;
      
      -- weitere Institutionen 3, ID: 0
      IF r.kd_institution_name_3 IS NOT NULL
           OR r.kd_institution_kundennummer_3 IS NOT NULL
           OR r.kd_institution_strasse_3 IS NOT NULL
           OR r.kd_institution_strassenr_3 IS NOT NULL
           OR r.kd_institution_plz_3 IS NOT NULL
           OR r.kd_institution_ort_3 IS NOT NULL
           OR r.kd_institution_vorwahl_3 IS NOT NULL
           OR r.kd_institution_telefonnummer_3 IS NOT NULL
           OR r.kd_institution_email_3 IS NOT NULL THEN

         uuid = generate_uuidgen();
	 rgid = create_rg_ansprechpartner_tbl(r.id, uuid);

	 UPDATE rg_ansprechpartner_tbl SET
	    ansprechpartner_typ = 0,
            ansprechpartner_name = r.kd_institution_name_3,
	    ansprechpartner_kundennummer = r.kd_institution_kundennummer_3,
	    ansprechpartner_strasse = r.kd_institution_strasse_3,
	    ansprechpartner_strassenr = r.kd_institution_strassenr_3,
            ansprechpartner_plz = r.kd_institution_plz_3,
            ansprechpartner_ort = r.kd_institution_ort_3,
	    ansprechpartner_vorwahl = r.kd_institution_vorwahl_3,
	    ansprechpartner_telefonnummer = r.kd_institution_telefonnummer_3,
	    ansprechpartner_email = r.kd_institution_email_3
	 WHERE id = rgid;
      END IF;


      -- Falls keine RGs angelegt wurden, eine neue erstellen
      SELECT COUNT(id) INTO rg_count FROM rg_ansprechpartner_tbl WHERE master_id  = r.id;
      IF rg_count = 0 THEN
      	uuid = generate_uuidgen();
        rgid = create_rg_ansprechpartner_tbl(r.id, uuid);
      END IF;

  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
SELECT insert_rg();
SET ROLE :db_adm;
DROP FUNCTION insert_rg();




-- Restore old function with its corresponding exception
CREATE OR REPLACE FUNCTION create_rg_ansprechpartner_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_ansprechpartner_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_ansprechpartner_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_ansprechpartner_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_ansprechpartner_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;



SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

-- ROLLBACK;
COMMIT;
